/***********************************************************************
par Silicium628

========================================================================
ATmega32 + afficheur LCD 5110 Nokia
Enregistreur d'octets rapide sur SDcard en mode RAW

************************************************************************/

/**
REMARQUE CONCERNANT L'UTILISATION DE CE PROGRAMME POUR LE CONTROLE DE LA MACHINE CNC :

	Mémorisation numérique (dans une SDcard) du tracé pour contôle sans tracé physique puis tracé à l'écran par un soft en Qt4.
	La fréquence max envoyée aux cartes EasyDriver en vitesse rapide est = 2500Hz soit 5000 évènements/s (puisque 2 moteurs)
	Si l'on veut mémoriser toutes les impulsions moteur envoyées, chaque évènement étant défini par 2bits (moteur , sens ),
	chaque octet peut mémoriser 8bits/2bits = 4 évènements, 
	5000/4 = 1250 octets/s (= 10 kbits/s)
	si on veut mémoriser 1/2h de tracé, il faut 1250 octets/s * 1800s = 2.25MB. -> la RAM du Mega2560 = 8kB soit 280 fois moins !!
	utiliser un processeur ARM ? un Raspberry Pi2 quad core ?
	Plus simplement cette acquisition externe par un ATmega8 connecté sur les sorties des signaux, qui mémorise dans une SDcard.
	1250 octets/s * 1800s / (512 octets/blocs) = un peu moins de 4400 blocs.
	(1250 oct/s) / (512 oct/blocs) =  2.44 blocs/s -> écriture de 1 bloc toutes les 0.41s 
	
	4400 blocs x (512 octets/bloc) = 2252800  octets = 2200 kB = 2MB.
	vitesse d'enregistrement max : les 2252800 octets peuvent être écrits en 24s soit 184 blocs/s ou 94 kB/s = 750 kbits/s (750kHz) soit 75 fois plus rapide que la vitesse requise ici.
	En pratique, et sans utiliser le double buffering, ce programme peut enregistrer à 22 kB/s (22kHz). On enregistre dans le buffer que l'on est en train de graver sur la SDcard...

	Le resultat enregistré sur la SDcard (non partitionnée !) est récupérable sous linux par la commande "dd" suivante:
	"sudo dd if=/dev/sdc of=nom_fichier bs=512 count=100"  en adaptant bien sûr "sdc" à la valeur correspondant réellement à la SDcard dans le système.
	On obtient un fichier lisible sous linux avec jeex puisque écrit en binaire.
	Ce fichier est interprétable (et traçable) par le soft en Qt4 associé à ce programme.
**/

/*
 ==================  PARAMETRES NUMERIQUE DE CE PROGRAMME ============================================================================================

(Les tempo sont mesurées à l'oscillo et/ou fréquencemètre) et non pas calculées exactement au simulateur
OCR0 = 10; -> un octet est présenté toutes les 45us c.a.d avec une fréquence de 22kHz (pour test avec acquisition interne par Timer0))
-Un enregistrement de 1 block (=512 octets) se fait en 8ms en moyenne avec quelques rares "coups" à 16ms (? deux blocs à la fois ????)
-Le buffer de 512 octet se remplit en 512 x 45us = 23ms
-Le temps de repos pendant lequel il n'y a pas d'écriture sur la carte = 23ms-8ms= 15ms; On voit très bien ce temps de 15ms à l'oscillo
-Chaque octet est enregistré sur la carte en 8ms/512 = 15us soit moins que la période d'arrivée des octets qui est rappelons le de 45us soit 3x plus longue.

CE QUI SE PASSE:
A l'instant où le buffer est plein, on dispose de 45us avant qu'un octet ne se présente à l'enregistrement en début du buffer.
Lorsque cet octet incident arrive, on a déjà enregistré les trois premier octets du buffer sur la carte, lorsque le deuxième octet arrive on a enregistré 6 octets
et ainsi de suite, de telle manière que l'adresse de stockage des octets entrants dans le buffer ne rattrape jamais celle des octets lus pour être expédié dans la SDcard.
Ceci es important de façon à ce que les octets arrivant n'écrasent pas des octets pas encore enregistrés: Le serpent ne doit pas se mordre la queue !
Il n'y a donc aucune perte de signal, tout ce qui arrive est enregistré sur la carte.

======================================================================================================================================================
//*/


//#define F_CPU 16000000


//#define ACQ_INTERNE
 
#include <avr/io.h>
#include <stdint.h> 
#include <stdlib.h>
#include "timeout.h"   // comprend : #define F_CPU 16000000UL  // 16 MHz
#include <avr/interrupt.h>
#include "LCD_5110-628b.cpp" // penser à paramétrer correctement les ports et bits popur le LCD dans le fichier "LCD_5110-628b.h"
#include <math.h>

#include "SDcard_628_2.c"

const char * version = "v:3.1";


#define PORT_led PORTD
#define bit_LED_verte 0b00010000
#define bit_LED_rouge 0b00100000


	uint16_t err;
	uint8_t octet_out;
	uint8_t n_evnt; // (=0..3) à mesure que l'octet_out se construit
	uint16_t adr_octet; // dans le block (=0..511)
	volatile uint32_t num_block;
	uint8_t compteur1;
	volatile uint16_t compteur2 = 0x0000;
	volatile uint8_t flag_ecrire_block = 0x00;  //  'volatile' afin qu'elle soit prise en compte par l'interruption
	volatile uint8_t c;
	volatile char string1[16+1] = "00000_dataRecord";
	uint8_t flag_acquisition;

// static uint8_t buffer[BLOCK_SIZE];  // BLOCK_SIZE est défini dans "sdCard.h"


void init_ports (void)
// 0 = entree, 1=sortie   ;   les 1 sur les pins en entrees activent les R de Pull Up (tirage à VCC)
{
	DDRB  = 0b10111111;	// PB6 en entrée (MISO)
	PORTB = 0b11101111;	// PB4=0 (CS)

// le pin_CS est définit dans le fichier SD_routines628.h	
	 
  DDRC = 	0b11111111;	
  PORTC = 0b00000000;

  DDRD =  0b11110000; // PortD[0..3] = entrées (int0=STEP moteur1, int1=STEP moteur2 , DIR moteur1 et DIR moteur1)
  PORTD = 0b00000000;
}
 

void InitINTs (void)
{
	
	GICR 		|=  0b11000000;  // gere les INTs - bit6  ->INT0 request - bit7  ->INT1 request;  voir page 68 du pdf ATmega8A
  MCUCR		|=  0b00001010;  // Falling edge of INT0 (et INT1) generates an interrupt request.  p:67 du pdf


	TCCR0 = 0;
	TCCR0  |= 0b00001000;	// Timer0 en mode CTC (Clear Timer on Compare Match): WGM00=1 (bit6) et WGM01=0 (bit3) voir p:74 et 80
	TCCR0  |= 0b00000011;	// Timer0 prescaller = f=clk/64.	0CS0[2,1,0] = 011 voir p:80 et p:82
	TIMSK  = 0b00000010;	// Timer/Counter0, Output Compare Match Interrupt Enable; Voir datasheet ATmega32 p:82	

	OCR0 = 10;	// Timer0 (freq = 250/n environ) : n=249->1kHz;  126->2kHz;    50->5kHz;   30->8kHz;   20 -> 12kHz;  10-> 22kHz;  5->41kHz;   2->83kHz;

}


void init_variables(void)
{
	flag_ecrire_block=0;
	flag_acquisition=0;
	err=0;
	octet_out=0;
	n_evnt=0; 
	adr_octet=0;
	num_block=1; // je reserve le bloc zero
	compteur1=0;
	compteur2=0;
}



void num2string(uint16_t nb_i, volatile char str_i[17])  // remarque: ce tableau de caractères est implicitement passé par adresse, son contenu sera donc changé chez l'appelant
{
/** RAPPEL :
	Lors du passage d'un tableau en paramètre d'une fonction, la conversion implicite a lieu, les tableaux sont donc toujours passés par variable, jamais par valeur.
	Il est donc faux d'utiliser des pointeurs pour les passer en paramètre, car le paramètre aurait le type pointeur de tableau.
	On ne modifierait pas le tableau, mais bel et bien le pointeur du tableau.	Le programme aurait donc de fortes chances de planter.
**/	
//surcharge les 5 premiers caratères de la chaine avec les caratères	représentant le nombre transmis
	uint8_t i, r;
	for (i=1; i<=5; i++) 
	{
		r=48 + nb_i % 10; // % c'est le modulo (reste de la division)
		nb_i /= 10; // quotient
		str_i[5-i]=r;  // surcharge le début de la ligne (les "0") avec le num
	}
}


void num2string_bin(uint16_t nb_i, volatile char str_i[17])  // remarque: ce tableau de caractères est implicitement passé par adresse, son contenu sera donc changé chez l'appelant
{
/** RAPPEL :
	Lors du passage d'un tableau en paramètre d'une fonction, la conversion implicite a lieu, les tableaux sont donc toujours passés par variable, jamais par valeur.
	Il est donc faux d'utiliser des pointeurs pour les passer en paramètre, car le paramètre aurait le type pointeur de tableau.
	On ne modifierait pas le tableau, mais bel et bien le pointeur du tableau.	Le programme aurait donc de fortes chances de planter.
**/	
//surcharge les 2 premiers caratères de la chaine avec le nombre trransmis  

	str_i[0] = nb_i >> 8;
	str_i[1] = nb_i & 0xFF;
}



void bufferise_octet(uint8_t octet_i)
{
// buffer est définit dans "sdCard_628.h"  c'est -> volatile uint8_t buffer[512];

	buffer[adr_octet] = octet_i;
	
	compteur1++;	if (compteur1 > 50) {compteur1 =0;   PORT_led  ^= bit_LED_verte; }
	adr_octet++;
	if (adr_octet > 511)
	{
		adr_octet =0;
		flag_ecrire_block = 1;  // l'écriture du block se fera dans la boucle principale, hors de l'INT donc.
	}
}

 

void complete_octet(uint8_t bibit)
{
	octet_out <<= 2; // décallage de deux bits à gauche, des zéros sont insérés à droite. Attention : si oubli du '=' pas de modif de la variable
	octet_out += bibit;

	n_evnt++;
	if (n_evnt > 3)
	{
		n_evnt=0;
		bufferise_octet(octet_out);
		octet_out=0;
	}
}



ISR (BADISR_vect)
{
// évite de planter si une int est enable et pas de procedure associée écrite (ce qui fait reseter l'ATmega)
}



ISR (INT0_vect) 
{
#ifndef ACQ_INTERNE
//interruption sur front descendant sur l'entree Int0
	if (flag_acquisition==1) // bloque lors de la phase d'init
	{
		uint8_t etat;
		etat = PIND & 0b00000001; 
		if (etat == 0)	{	complete_octet(0b01);	}	else {	complete_octet(0b00);	}
	}
#endif
}


ISR (INT1_vect) 
{
#ifndef ACQ_INTERNE

//interruption sur front descendant sur l'entree Int1
	if (flag_acquisition==1) // bloque lors de la phase d'init
	{
		uint8_t etat;
		etat = PIND & 0b00000010; 
		if (etat == 0)	{	complete_octet(0b11);	}	else {	complete_octet(0b10);	}
	}

#endif	
}



ISR (TIMER0_COMP_vect) // acquisition "interne "pour tests
{
#ifdef ACQ_INTERNE
	if (flag_acquisition==1) // bloque lors de la phase d'init
	{
		PORT_led |= bit_LED_verte;
//complete_octet(compteur2 & 0b00000011); // (compteur2 && 0b00000011) = [00..01..10..11..]

//		char string1[16+1] = "00000_dataRecord";

		//num2string(compteur2, string1);
		num2string_bin(compteur2, string1);
		
		bufferise_octet(string1[c]);
		c++;
		if (c > 15)
		{
			c = 0;
			compteur2++;
		}
		PORT_led &= ~bit_LED_verte;
	}
#endif		
}




void raz_buffer()
{
	uint16_t i;
	for (i=0; i<=512; i++)
	{
		buffer[i] = '-';
	}
}




void fill_buffer_Char(uint16_t num) // num est le numéro de block qui sera écrit en début de ligne
{
	uint8_t c, r;
	uint16_t k;

	//--------------------"1234567890123456"
//char string1[16+1] = "00000solarImpuls";
//char string1[16+1] = "00000silicium628";
//char string1[16+1] = "00000tagadapouet";
//char string1[16+1] = "00000tartalacrem";
//char string1[16+1] = "00000hello World";
//char string1[16+1] = "00000doremifasol";
//char string1[16+1] = "00000_dataRecord";
char string1[16+1] = "00000===========";

	
	num2string(num, string1);

	c = 0;
	for (k=0; k<=512; k++)
	{
		buffer[k] = string1[c]; 
		c++;
		if (c > 15) { c = 0; }
	}
}



void fill_buffer_Int()
{
	uint16_t i;
	uint8_t c;
	c = 0;
	for (i=0; i<=512; i++)
	{
		buffer[i] = c;
		c++;  // c = 1 octet = 0..255
	}
}



int main (void)
{
	uint16_t i;
	uint32_t adresse1;
	uint8_t ok=0;


	unsigned char option, error, data, FAT32_active;
	
	init_variables(); 
	init_ports();
	InitINTs();
	lcd_init();
	
	lcd_clear();
	_delay_ms(300);

	lcd_puts("SDcrd Recorder");
	lcd_goto_LC(1, 0);
	lcd_puts(version);
	_delay_ms(1000);
	lcd_clear();
	
	sei();

	cardType = 0;

	lcd_puts("init...");
	lcd_goto_LC(1,0);

	SPI_SetSlow();

	for (i=0; i<10; i++)
	{
		error = SD_init();
		if(!error) break;
	}

	if(error)
	{
		if(error == 1)	{	lcd_puts("SD no detect");	}
		if(error == 2) 	{	lcd_puts("SD Init fail");	}
		lcd_goto_LC(2, 0);
    lcd_puts("faire un reset");
		while(1) { ; }  
	}

	switch (cardType)
	{
		case 1:lcd_puts("Std Card 1.x");
		break;
		case 2:lcd_puts("SDHC Card ok");
		break;
		case 3:lcd_puts("Std Card 2.x");
		break;
		default:lcd_puts("SDcard ?");
		break; 
	}

	SPI_SetFast();

	_delay_ms(1000);
	lcd_clear();

	lcd_goto_LC(1, 0);
	lcd_puts("Effacement...");

	uint16_t n;

	fill_buffer_Int();
	SD_writeSingleBlock(0); // remplit le bloc zéro avec des entiers [0..511] On pourra y enregistrer tout autre chose, comme la longueur de l'enregistrement par exemple

	for (n=1; n<2000; n++)  //200 pour tests;  ensuite 2000 ou plus... à voir
	{
		fill_buffer_Char(n);
//PORT_led  |= bit_LED_verte;
		SD_writeSingleBlock(n);
//PORT_led  &= ~bit_LED_verte;
		lcd_goto_LC(2,0);
		lcd_aff_nb (n, 4, 0);
	}


	lcd_goto_LC(2, 0);
	lcd_puts("Erase SDcrd ok");
	_delay_ms(1000);
	lcd_goto_LC(4, 1);
	lcd_puts("RECORDING...");

	_delay_ms(100);

	num_block=1;
	flag_acquisition=1;
	
	while(1)
	{
		if (flag_ecrire_block == 1)
		{
PORT_led |= bit_LED_rouge;
//_delay_us(200);
					
//flag_acquisition=0;
		SD_writeSingleBlock(num_block);  // écriture du buffer plein (qui n'est plus en cours) sur la SDcard.
		num_block++;
//flag_acquisition=1;
			
			
PORT_led &= ~bit_LED_rouge;			

			flag_ecrire_block = 0;
		}
		//_delay_ms(300);
	}
}

