//**************************************************************
// ****** FUNCTIONS FOR SD RAW DATA TRANSFER *******
//**************************************************************
	/**fichier d'origine = SD_routines.c**/
//Controller: ATmega32 (Clock: 8 Mhz-internal)
//Compiler	: AVR-GCC (winAVR with AVRStudio)
//Version 	: 2.3
//Author	: CC Dharmani, Chennai (India)
//			  www.dharmanitech.com
//Date		: 08 May 2010
//**************************************************************

/**
	gravement modifié par Silicium628
	2015-07-23
**/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "SPI_628.c"
#include "SDcard_628_2.h"

uint8_t i_buf; // =0 ou 1

//******************************************************************
uint8_t SD_init(void)
{
	uint8_t i, reponse, SD_version;
	uint16_t retry=0 ;


	 for(i=0;i<10;i++)
				SPI_transmit(0xff);   //80 clock pulses spent before sending the first command
		
	SD_CS_ASSERT;


	reponse = 0;
	while(reponse != 0x01)
	{
		reponse = SD_sendCommand(GO_IDLE_STATE, 0); //send 'reset & go idle' command
		retry++;
		if(retry>0x20) {  return 1; }  //time out, card not detected
	} 

	SD_CS_DEASSERT;
	SPI_transmit (0xff);
	SPI_transmit (0xff);

	retry = 0;

	SD_version = 2; //default set to SD compliance with ver2.x; this may change after checking the next command

	reponse = 0;				
	while(reponse != 0x01)
	{
	reponse = SD_sendCommand(SEND_IF_COND,0x000001AA); //Check power supply status, mendatory for SDHC card
	retry++;
	if(retry>0xfe) 
		 {
			SD_version = 1;
			cardType = 1;
			break;
		 } //time out
	}

	retry = 0;

	do
	{
		reponse = SD_sendCommand(APP_CMD,0); //CMD55, must be sent before sending any ACMD command
		reponse = SD_sendCommand(SD_SEND_OP_COND,0x40000000); //ACMD41
		retry++;
		if(retry>0xfe) 	{return 2;  } //time out, card initialization failed
	} while(reponse != 0x00);


	retry = 0;
	SDHC_flag = 0;

	if (SD_version == 2)
	{ 
		do
		{
			reponse = SD_sendCommand(READ_OCR,0);
			retry++;
			if(retry>0xfe) { cardType = 0;	 break;	} //time out
		} while(reponse != 0x00);

		 if(SDHC_flag == 1) { cardType = 2; }
		 else cardType = 3;
	}
	//SD_sendCommand(CRC_ON_OFF, OFF); //disable CRC; deafault - CRC disabled in SPI mode
	//SD_sendCommand(SET_BLOCK_LEN, 512); //set block size to 512; default size is 512
	return 0; //successful
}


//******************************************************************
uint8_t SD_sendCommand(uint8_t cmd, uint32_t arg)
{
	uint8_t reponse, retry=0, status;

	//SD card accepts byte address while SDHC accepts block address in multiples of 512
	//so, if it's SD card we need to convert block address into corresponding byte address by 
	//multipying it with 512. which is equivalent to shifting it left 9 times
	//following 'if' loop does that

	if(SDHC_flag == 0)		
	{
		if(cmd==READ_SINGLE_BLOCK || cmd==READ_MULTIPLE_BLOCKS  || cmd==WRITE_SINGLE_BLOCK || cmd==WRITE_MULTIPLE_BLOCKS || cmd==ERASE_BLOCK_START_ADDR || cmd==ERASE_BLOCK_END_ADDR ) 
		{
			arg = arg << 9;
		}
	}  
	SD_CS_ASSERT;

	SPI_transmit(cmd | 0x40); //send command, first two bits always '01'
	SPI_transmit(arg>>24);
	SPI_transmit(arg>>16);
	SPI_transmit(arg>>8);
	SPI_transmit(arg);

//to send correct CRC for CMD8 (CRC=0x87) & CMD0 (CRC=0x95) :
	if(cmd == SEND_IF_COND)	{ SPI_transmit(0x87); }	else  { SPI_transmit(0x95); }

	while((reponse = SPI_receive()) == 0xFF)  { if(retry++ > 0xFE) break; } //attend reponse

	if(reponse == 0x00 && cmd == 58)  //test reponse du CMD58
	{
		status = SPI_receive() & 0x40;     //first byte of the OCR register (bit 31:24)
		if(status == 0x40) SDHC_flag = 1;  //we need it to verify SDHC card
		else SDHC_flag = 0;

		SPI_receive(); //remaining 3 bytes of the OCR register are ignored here
		SPI_receive(); //one can use these bytes to check power supply limits of SD
		SPI_receive(); 
	}
	SPI_receive(); //extra 8 CLK
	SD_CS_DEASSERT;
	return reponse; //return state
}

//*****************************************************************
//Function	: to erase specified no. of blocks of SD card
//Arguments	: none
//return	: uint8_t; will be 0 if no error,
// 			  otherwise the reponse byte will be sent
//*****************************************************************
uint8_t SD_erase (uint32_t startBlock, uint32_t totalBlocks)
{
	uint8_t reponse;

	reponse = SD_sendCommand(ERASE_BLOCK_START_ADDR, startBlock); //send starting block address
	if(reponse != 0x00) { return reponse; }//check for SD status: 0x00 - OK (No flags set)
	
	reponse = SD_sendCommand(ERASE_BLOCK_END_ADDR,(startBlock + totalBlocks - 1)); //send end block address
	if(reponse != 0x00) { return reponse; }

	reponse = SD_sendCommand(ERASE_SELECTED_BLOCKS, 0); //erase all selected blocks
	if(reponse != 0x00) { return reponse; }

	return 0; //normal return
}

//******************************************************************
//Function	: to read a single block from SD card
//Arguments	: none
//return	: uint8_t; will be 0 if no error,
// 			  otherwise the reponse byte will be sent
//******************************************************************
uint8_t SD_readSingleBlock(uint32_t startBlock)
{
	uint8_t reponse;
	uint16_t i, retry=0;

	reponse = SD_sendCommand(READ_SINGLE_BLOCK, startBlock); //read a Block command
	if(reponse != 0x00) { return reponse; } //check for SD status: 0x00 - OK (No flags set)

	SD_CS_ASSERT;

	retry = 0;
	while(SPI_receive() != 0xFE) { if(retry++ > 0xFFFE){SD_CS_DEASSERT; return 1;} }	//wait for start block token 0xfe (0x11111110) ; return if time-out

	for(i=0; i<512; i++) { buffer[i] = SPI_receive(); } //read 512 bytes
	
	SPI_receive(); //receive incoming CRC (16-bit), CRC is ignored here
	SPI_receive();

	SPI_receive(); //extra 8 clock pulses
	SD_CS_DEASSERT;
	return 0;
}

//******************************************************************
//Function	: to write to a single block of SD card
//Arguments	: none
//return	: uint8_t; will be 0 if no error,
// 			  otherwise the reponse byte will be sent
//******************************************************************
uint8_t SD_writeSingleBlock(uint32_t startBlock)
{
	uint8_t reponse;
	uint16_t i, retry=0;

	 reponse = SD_sendCommand(WRITE_SINGLE_BLOCK, startBlock); //write a Block command
	 if(reponse != 0x00) { return reponse; }//check for SD status: 0x00 - OK (No flags set)

	SD_CS_ASSERT;
	SPI_transmit(0xFE);     //Send start block token 0xfe (0x11111110)

	for(i=0; i<512; i++)  { SPI_transmit(buffer[i]); }   //send 512 bytes data

	SPI_transmit(0xFF);     //transmit dummy CRC (16-bit), CRC is ignored here
	SPI_transmit(0xFF);

	reponse = SPI_receive();
	if( (reponse & 0x1F) != 0x05)  { SD_CS_DEASSERT; return reponse; } //reponse= 0xXXX0AAA1 ; AAA='010' - data accepted ;AAA='101'-data rejected due to CRC error; AAA='110'-data rejected due to write error


	while(!SPI_receive()) { if(retry++ > 0xFFFE) { SD_CS_DEASSERT; return 1; } }   //wait for SD card to complete writing and get idle
	
	SD_CS_DEASSERT;
	SPI_transmit(0xFF);   //just spend 8 clock cycle delay before reasserting the CS line
	SD_CS_ASSERT;         //re-asserting the CS line to verify if card is still busy

	while(!SPI_receive()) { if(retry++ > 0xFFFE){SD_CS_DEASSERT; return 1;} } //wait for SD card to complete writing and get idle
		 
	SD_CS_DEASSERT;

	
	return 0;
}


#ifndef FAT_TESTING_ONLY

//***************************************************************************
//Function	: to read multiple blocks from SD card & send every block to UART
//Arguments	: none
//return	: uint8_t; will be 0 if no error,
// 			  otherwise the reponse byte will be sent
//****************************************************************************
uint8_t SD_readMultipleBlock (uint32_t startBlock, uint32_t totalBlocks)
{
	uint8_t reponse;
	uint16_t i, retry=0;

	retry = 0;

	reponse = SD_sendCommand(READ_MULTIPLE_BLOCKS, startBlock); //write a Block command
		
	if(reponse != 0x00) return reponse; //check for SD status: 0x00 - OK (No flags set)

	SD_CS_ASSERT;

	while( totalBlocks )
	{
		retry = 0;
		while(SPI_receive() != 0xFE)  { if(retry++ > 0xfffe){SD_CS_DEASSERT; return 1;}  } //wait for start block token 0xfe (0x11111110);  return if time-out

		for(i=0; i<512; i++) { buffer[i] = SPI_receive(); }//read 512 bytes
	
		SPI_receive(); //receive incoming CRC (16-bit), CRC is ignored here
		SPI_receive();

		SPI_receive(); //extra 8 cycles
		//TX_NEWLINE;
		//transmitString_F(PSTR(" --------- "));
		//TX_NEWLINE;

		for(i=0; i<512; i++) //send the block to UART
		{
			if(buffer[i] == '~') break;
			transmitByte ( buffer[i] );
		}

		//TX_NEWLINE;
		//transmitString_F(PSTR(" --------- "));
		//TX_NEWLINE;
		totalBlocks--;
	}

	SD_sendCommand(STOP_TRANSMISSION, 0); //command to stop transmission
	SD_CS_DEASSERT;
	SPI_receive(); //extra 8 clock pulses

	return 0;
}

//***************************************************************************
//Function: to receive data from UART and write to multiple blocks of SD card
//Arguments: none
//return: uint8_t; will be 0 if no error,
// otherwise the reponse byte will be sent
//****************************************************************************
uint8_t SD_writeMultipleBlock(uint32_t startBlock, uint32_t totalBlocks)
{
	uint8_t reponse, data;
	uint16_t i, retry=0;
	uint32_t blockCounter=0, size;

	reponse = SD_sendCommand(WRITE_MULTIPLE_BLOCKS, startBlock); //write a Block command

	if(reponse != 0x00) return reponse; //check for SD status: 0x00 - OK (No flags set)

	SD_CS_ASSERT;

	TX_NEWLINE;
	transmitString_F(PSTR(" Enter text (End with ~): "));
	TX_NEWLINE;

	while( blockCounter < totalBlocks )
	{
		i=0;
		do
		{
			data = receiveByte();
			if(data == 0x08)	//'Back Space' key pressed
			{ 
				if(i != 0)
				{ 
					transmitByte(data);
					transmitByte(' '); 
					transmitByte(data); 
					i--; 
					size--;
			} 
			continue;     
			}
			transmitByte(data);
			buffer[i++] = data;
			if(data == 0x0D)
			{
				transmitByte(0x0A);
				buffer[i++] = 0x0A;
			}
			if(i == 512) break;
			}while (data != '~');

			//TX_NEWLINE;
		 transmitString_F(PSTR(" ---- "));
			//TX_NEWLINE;

		 SPI_transmit(0xFC); //Send start block token 0xfc (0x11111100)

		 for(i=0; i<512; i++) //send 512 bytes data
			 SPI_transmit( buffer[i] );

		 SPI_transmit(0xFF); //transmit dummy CRC (16-bit), CRC is ignored here
		 SPI_transmit(0xFF);

		 reponse = SPI_receive();
		 if( (reponse & 0x1F) != 0x05) //reponse= 0xXXX0AAA1 ; AAA='010' - data accepted
		 {                              //AAA='101'-data rejected due to CRC error
				SD_CS_DEASSERT;             //AAA='110'-data rejected due to write error
				return reponse;
		 }

		 while(!SPI_receive()) //wait for SD card to complete writing and get idle
			 if(retry++ > 0xFFFE){SD_CS_DEASSERT; return 1;}

		 SPI_receive(); //extra 8 bits
		 blockCounter++;
	}

	SPI_transmit(0xFD); //send 'stop transmission token'

	retry = 0;

	while(!SPI_receive()) //wait for SD card to complete writing and get idle
		 if(retry++ > 0xFFFE){SD_CS_DEASSERT; return 1;}

	SD_CS_DEASSERT;
	SPI_transmit(0xFF); //just spend 8 clock cycle delay before reasserting the CS signal
	SD_CS_ASSERT; //re assertion of the CS signal is required to verify if card is still busy

	while(!SPI_receive()) //wait for SD card to complete writing and get idle
		 if(retry++ > 0xFFFE){SD_CS_DEASSERT; return 1;}
	SD_CS_DEASSERT;

	return 0;
}
//*********************************************

#endif

//******** END ****** www.dharmanitech.com *****
