/**
 * @file       sdCard.h
 * @brief      read/write in a sdcard throught SPI port of an atmel 8nits microcontroler
 * 
 * @author     Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 * @date       2009/06/27
 */

/*
gravement modifié par Silicium628  ;)
2015-07-03

Fonctionne avec une SDcard de 1GB mais pas avec des SDcards HC classe4 de 4GB...
il reste à déterminer pourquoi, vu que les cartes de 1 GB non HC deviennent introuvales.
*/

#include <stdint.h> 

#ifndef __SDCARD_H__
#define __SDCARD_H__
// command the following lines if you just want read/write sdCard functions (to save program disk space)
#define SD_CARD_REGISTER_COMMANDS

// #include "util.h"

#define BLOCK_SIZE  512 //  block size of the SDCard. Always 512 bytes
#define SD_CARD_MAX_BLOCK   0x300000 // maximum number of blocks for a 2Go SDCard = Capacity / BLOCK_SIZE = 0x400000

typedef uint32_t sdCardAddr;

static uint8_t buffer[BLOCK_SIZE];  

volatile unsigned long startBlock, totalBlocks; 
volatile uint8_t SDHC_flag, cardType;


/**
	Initializes the sdCard capture module (setup ATmega SPI port)
*/
void sdInit_ports();

/**
	This function returns the SD error code after a sdXXX function return false
*/
uint16_t sdGetError();

/**
	Detect and initialize the SDcard
	true  if the card is detected
	false if the card is not detected. Use sdGetError to get the error code.
*/
uint8_t sdInitCard();


/**
Writes BLOCK_SIZE bytes in the sdcard at the specified blockId
lockId the number of the block to write: 0, 1, ... cardCapacity/BLOCK_SIZE
buffer an array of at least BLOCK_SIZE bytes containing the data to write in the sdCard

return  true  if the block has been written in the sdCard
return false if the block has not been written completely. Use sdGetError to get the error code.
 */
uint8_t sdWriteBlock(sdCardAddr blockId);


/**
Read BLOCK_SIZE bytes from the sdcard at the specified blockId

param blockId the number of the block to read: 0, 1, ... cardCapacity/BLOCK_SIZE
param buffer an array of at least BLOCK_SIZE bytes containing the data to write in the sdCard

return true  if the block has been read from the sdCard
return false if the block has not been read completely. Use sdGetError to get the error code.
 */
uint8_t sdReadBlock(sdCardAddr blockId);

// ===========================================================================
// Functions to get sd card registers
// ===========================================================================

#ifdef SD_CARD_REGISTER_COMMANDS


/** 
Retrieve the 16 bytes Card Specific Data register
param buffer an array of at least 16 bytes that will be fill with the register data
return true on success, false in the other case
 */
uint8_t sdGetCSD(unsigned char* buffer);


/** 
Retrieve the 16 bytes Card Identification register
param buffer an array of at least 16 bytes that will be fill with the register data
return true on success, false in the other case
 */
uint8_t sdGetCID(unsigned char* buffer);



/** 
Retrieve the 4 bytes Operating Conditions Register
param buffer an array of at least 4 bytes that will be fill with the register data
return true on success, false in the other case
 */
uint8_t sdGetOCR(unsigned char* buffer);



/** 
Retrieve the 2 bytes Status Register
param buffer an array of at least 2 bytes that will be fill with the register data
return true on success, false in the other case
 */
uint8_t sdGetStatus(unsigned char* buffer);

#endif // SD_CARD_REGISTER_COMMANDS

#endif

